==> dropwizard-revolver-1.3.7-25-20190401.052926-15-sources.jar: io/dropwizard/revolver/optimizer/OptimizerConfigUpdater.java <==
package io.dropwizard.revolver.optimizer;

import com.google.common.collect.Maps;
import io.dropwizard.revolver.RevolverBundle;
import io.dropwizard.revolver.core.config.RevolverConfig;
import io.dropwizard.revolver.core.config.hystrix.ThreadPoolConfig;
import io.dropwizard.revolver.optimizer.config.OptimizerConcurrencyConfig;
import io.dropwizard.revolver.optimizer.config.OptimizerConfig;
import io.dropwizard.revolver.optimizer.utils.OptimizerUtils;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.msgpack.jackson.dataformat.Tuple;

import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

/***
 Created by nitish.goyal on 29/03/19
 ***/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class OptimizerConfigUpdater implements Runnable {

    private RevolverConfig revolverConfig;
    private OptimizerConfig optimizerConfig;

    @Override
    public void run() {

        Map<String, OptimizerAggregatedMetrics> optimizerAggregatedMetricsMap = Maps.newHashMap();
        Map<Tuple<Long, String>, OptimizerMetrics> metricsCache = OptimizerMetricsCache.getCache();
        if(metricsCache.isEmpty()) {
            return;
        }
        metricsCache.forEach((tuple, optimizerMetrics) -> {
            if(optimizerAggregatedMetricsMap.get(tuple.second()) == null)
                optimizerAggregatedMetricsMap.put(tuple.second(), OptimizerAggregatedMetrics.builder()
                        .pool(tuple.second())
                        .metricsMaxValueMap(Maps.newHashMap())
                        .build());

            OptimizerAggregatedMetrics optimizerAggregatedMetrics = optimizerAggregatedMetricsMap.get(tuple.second());
            Map<String, Number> aggregatedMetricsValues = optimizerAggregatedMetrics.getMetricsMaxValueMap();

            optimizerMetrics.getMetrics()
                    .forEach((metric, value) -> {
                        if(aggregatedMetricsValues.get(metric) == null || aggregatedMetricsValues.get(metric)
                                                                                  .intValue() < value.intValue()) {
                            aggregatedMetricsValues.put(metric, value);
                        }
                    });

        });

        updateRevolverConfig(optimizerAggregatedMetricsMap);

    }

    private void updateRevolverConfig(Map<String, OptimizerAggregatedMetrics> optimizerAggregatedMetricsMap) {
        AtomicBoolean configUpdated = new AtomicBoolean();
        revolverConfig.getServices()
                .forEach(revolverServiceConfig -> {
                    if(revolverServiceConfig.getThreadPoolGroupConfig() == null) {
                        return;
                    }
                    revolverServiceConfig.getThreadPoolGroupConfig()
                            .getThreadPools()
                            .forEach(threadPoolConfig -> {
                                updatedPoolSettings(threadPoolConfig, optimizerAggregatedMetricsMap, configUpdated);
                            });
                });
        if(configUpdated.get()) {
            RevolverBundle.loadServiceConfiguration(revolverConfig);
        }
    }

    private void updatedPoolSettings(ThreadPoolConfig threadPoolConfig,
                                     Map<String, OptimizerAggregatedMetrics> optimizerAggregatedMetricsMap, AtomicBoolean configUpdated) {

        OptimizerAggregatedMetrics optimizerAggregatedMetrics = optimizerAggregatedMetricsMap.get(threadPoolConfig.getThreadPoolName());

        if(optimizerAggregatedMetrics == null) {
            return;
        }

        updateConcurrencySetting(threadPoolConfig, optimizerAggregatedMetrics, configUpdated);

    }

    private void updateConcurrencySetting(ThreadPoolConfig threadPoolConfig, OptimizerAggregatedMetrics optimizerAggregatedMetrics,
                                          AtomicBoolean configUpdated) {
        if(optimizerAggregatedMetrics.getMetricsMaxValueMap()
                   .get(OptimizerUtils.ROLLING_MAX_ACTIVE_THREADS) == null) {
            return;
        }
        OptimizerConcurrencyConfig concurrencyConfig = optimizerConfig.getConcurrencyConfig();
        int maxRollingActiveThreads = optimizerAggregatedMetrics.getMetricsMaxValueMap()
                .get(OptimizerUtils.ROLLING_MAX_ACTIVE_THREADS)
                .intValue();

        int concurrency = threadPoolConfig.getConcurrency();

        if(maxRollingActiveThreads > concurrency * concurrencyConfig.getMaxThreshold()) {
            concurrency = (int)(concurrency * concurrencyConfig.getIncreaseBy());
            configUpdated.set(true);
        } else if(maxRollingActiveThreads < concurrency * concurrencyConfig.getMinThreshold()) {
            concurrency = (int)(concurrency * concurrencyConfig.getDecreaseBy());
            configUpdated.set(true);
        }
        threadPoolConfig.setConcurrency(concurrency);
    }
}

==> dropwizard-revolver-1.3.7-25-20190401.064334-16-sources.jar: io/dropwizard/revolver/optimizer/OptimizerConfigUpdater.java <==
package io.dropwizard.revolver.optimizer;

import com.google.common.collect.Maps;
import io.dropwizard.revolver.RevolverBundle;
import io.dropwizard.revolver.core.config.RevolverConfig;
import io.dropwizard.revolver.core.config.hystrix.ThreadPoolConfig;
import io.dropwizard.revolver.optimizer.config.OptimizerConcurrencyConfig;
import io.dropwizard.revolver.optimizer.config.OptimizerConfig;
import io.dropwizard.revolver.optimizer.utils.OptimizerUtils;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.msgpack.jackson.dataformat.Tuple;

import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

/***
 Created by nitish.goyal on 29/03/19
 ***/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Slf4j
public class OptimizerConfigUpdater implements Runnable {

    private RevolverConfig revolverConfig;
    private OptimizerConfig optimizerConfig;

    @Override
    public void run() {

        Map<String, OptimizerAggregatedMetrics> optimizerAggregatedMetricsMap = Maps.newHashMap();
        Map<Tuple<Long, String>, OptimizerMetrics> metricsCache = OptimizerMetricsCache.getCache();
        if(metricsCache.isEmpty()) {
            return;
        }
        metricsCache.forEach((tuple, optimizerMetrics) -> {
            if(optimizerAggregatedMetricsMap.get(tuple.second()) == null)
                optimizerAggregatedMetricsMap.put(tuple.second(), OptimizerAggregatedMetrics.builder()
                        .pool(tuple.second())
                        .metricsMaxValueMap(Maps.newHashMap())
                        .build());

            OptimizerAggregatedMetrics optimizerAggregatedMetrics = optimizerAggregatedMetricsMap.get(tuple.second());
            Map<String, Number> aggregatedMetricsValues = optimizerAggregatedMetrics.getMetricsMaxValueMap();

            optimizerMetrics.getMetrics()
                    .forEach((metric, value) -> {
                        if(aggregatedMetricsValues.get(metric) == null || aggregatedMetricsValues.get(metric)
                                                                                  .intValue() < value.intValue()) {
                            log.error("Inputting into aggregated value : " + value.intValue() + ", for metric : " + metric + ", for key :" +
                                      " " + tuple.second());
                            aggregatedMetricsValues.put(metric, value);
                        }
                    });

        });
        updateRevolverConfig(optimizerAggregatedMetricsMap);

    }

    private void updateRevolverConfig(Map<String, OptimizerAggregatedMetrics> optimizerAggregatedMetricsMap) {
        AtomicBoolean configUpdated = new AtomicBoolean();
        revolverConfig.getServices()
                .forEach(revolverServiceConfig -> {
                    if(revolverServiceConfig.getThreadPoolGroupConfig() == null) {
                        return;
                    }
                    revolverServiceConfig.getThreadPoolGroupConfig()
                            .getThreadPools()
                            .forEach(threadPoolConfig -> {
                                updatedPoolSettings(threadPoolConfig, optimizerAggregatedMetricsMap, configUpdated);
                            });
                });
        if(configUpdated.get()) {
            log.error("Updating revolver config");
            RevolverBundle.loadServiceConfiguration(revolverConfig);
        }
    }

    private void updatedPoolSettings(ThreadPoolConfig threadPoolConfig,
                                     Map<String, OptimizerAggregatedMetrics> optimizerAggregatedMetricsMap, AtomicBoolean configUpdated) {

        OptimizerAggregatedMetrics optimizerAggregatedMetrics = optimizerAggregatedMetricsMap.get(threadPoolConfig.getThreadPoolName());

        if(optimizerAggregatedMetrics == null) {
            return;
        }
        updateConcurrencySetting(threadPoolConfig, optimizerAggregatedMetrics, configUpdated);

    }

    private void updateConcurrencySetting(ThreadPoolConfig threadPoolConfig, OptimizerAggregatedMetrics optimizerAggregatedMetrics,
                                          AtomicBoolean configUpdated) {
        if(optimizerAggregatedMetrics.getMetricsMaxValueMap()
                   .get(OptimizerUtils.ROLLING_MAX_ACTIVE_THREADS) == null) {
            return;
        }
        OptimizerConcurrencyConfig concurrencyConfig = optimizerConfig.getConcurrencyConfig();
        int maxRollingActiveThreads = optimizerAggregatedMetrics.getMetricsMaxValueMap()
                .get(OptimizerUtils.ROLLING_MAX_ACTIVE_THREADS)
                .intValue();
        int concurrency = threadPoolConfig.getConcurrency();

        if(maxRollingActiveThreads > concurrency * concurrencyConfig.getMaxThreshold()) {
            concurrency = (int)(concurrency * concurrencyConfig.getIncreaseBy());
            configUpdated.set(true);
        } else if(maxRollingActiveThreads < concurrency * concurrencyConfig.getMinThreshold()) {
            concurrency = (int)(concurrency * concurrencyConfig.getDecreaseBy());
            configUpdated.set(true);
        }
        log.error("Updating concurrency for pool : " + threadPoolConfig.getThreadPoolName() + " to value : " + concurrency + " from : " +
                  threadPoolConfig.getConcurrency() + ", maxRollingActiveThreads : " + maxRollingActiveThreads);
        threadPoolConfig.setConcurrency(concurrency);
    }
}
